// Copyright 2002 by SAS Institute Inc.
using System;
using System.Collections;
using System.Windows.Forms;
using System.Runtime.InteropServices;

/// <summary>
///This namespace name is used in conjunction with the UpdateTask class name to form the ProgID
///that EG will use to load this addin. The progid will be "EGUpdateAddIn.UpdateTask"
///This project is a Class Library project which means it generates a DLL. 
///The Register for COM Interop project setting must be set to true.
///In order for this task to work the following conditions must be met
///1. The .net runtime must be installed on the client machine
///2. The two dlls that this project creates ( egupdateaddin.dll, interop.sasaiobjlib.dll ) must
///be copied to the client machine and either installed in the GAC ( Global Assemble Cache ), or 
///copied to the same directory as Enterprise guide and registered with regasm.
///3. Enterprise Guide 2.0 must be installed on the client machine.
/// </summary>
namespace EGUpdateAddIn
{
	/// <summary>
	///these are the properties that get saved by EG, and used by this AddIn to recreate
	///the users choices.
	/// </summary>
	enum Properties { ColumnName, ReplaceWithList, ElseSetting, ElseValue, Code };
	/// <summary>
	//this enum corresponds to the users choice of what to do in the case that 
	//none of the replace conditions are met.
	//Missing means replace with a missing value
	//CurrentValue means leave the value as is
	//SpecifiedValue means replace with the specified value
	/// </summary>
	enum ElseSetting { Missing, CurrentValue, SpecifiedValue };
	
	/// <summary>
	/// This is the class that eg will use to comunicate with.
	/// must implement the SASAIOBJLib interfaces.
	/// </summary>
	[Guid("A22A4802-32CA-4d2d-A72E-F2CA88C085D2")]
	public class UpdateTask : SASAIOBJLib.SEGAddIn, SASAIOBJLib.SEGTask, SASAIOBJLib.SEGTaskInfo
	{
		//number of properties that are to be saved in the EG Project file
		const int numProps = 5;
		//this will be pass to us by EG, and represents the EG App
		private SASAIOBJLib.SEGApplication app;
		//the code generated by the UpdateForm
		private string Code;		
		//the name of the column that the values will be updated in
		private string strColName;
		//list of conditions in which to replace the values
		private string strReplaceList;
		//if the eElseSetting value is set to SpecifiedValue, this hold that value
		private string strElseValue;
		//else condition setting
		private ElseSetting eElseSetting;

		//constructor, will get called when EG Creates an instance of this task
		public UpdateTask()
		{
			//initial variables
			eElseSetting = ElseSetting.CurrentValue;
			strColName = "";
			strReplaceList = "";
		}

		/// <summary>
		///The rest of these methods are the interface methods called by EG to control this task
		///please see the EG AddIn documentation for details
		/// </summary>
		public void Connect( SASAIOBJLib.SEGApplication Application, int cookie, ref int Success )
		{
			Success = (int) SASAIOBJLib.APPCONNECTION.APP_CONNECTION_SUCCESS;
			this.app = Application;
		}

		public void Disconnect()
		{
			this.app = null;			
		}

		public void GetName( ref string Name )
		{
			Name = "Update Dataset";
		}

		public void GetDescription( ref string Description )
		{
			Description = "C# AddIn Sample: Update Dataset Using Proc SQL";
		}
		
		public void Initialize( ref int Success )
		{
			Success = ( int ) SASAIOBJLib.APPINITIALIZE.APP_INITIALIZE_SUCCESS;
		}
		
		public void Show( int ParentWindowHandle, ref int Status )
		{			
			Status = (int)SASAIOBJLib.APPSHOWSTATUS.APP_SHOW_CANCELED;
			UpdateForm uf = new UpdateForm();
			SASAIOBJLib.SEGData data = app.GetActiveData();
			if ( data != null )
			{
				uf.Data = data;
				uf.strColumnName = strColName;
				uf.strElseValue = strElseValue;
				uf.ReplaceList = strReplaceList;
				uf.eElse = this.eElseSetting;
				uf.ShowDialog();
				Status = uf.Status;
				if ( Status != (int)SASAIOBJLib.APPSHOWSTATUS.APP_SHOW_CANCELED )
				{					
					Code = uf.Code;		
					strColName = uf.strColumnName;
					strElseValue = uf.strElseValue;
					strReplaceList = uf.ReplaceList;
					eElseSetting = uf.eElse;
				}

				uf.Data = null;
			}
		}

		public void Terminate()
		{			
		}

		public void GetPropertyList( ref System.Array KeyArray )
		{
			KeyArray = new string[numProps];
			KeyArray.SetValue(Properties.ColumnName.ToString(),0);
			KeyArray.SetValue(Properties.ReplaceWithList.ToString(),1);
			KeyArray.SetValue(Properties.ElseSetting.ToString(),2);
			KeyArray.SetValue(Properties.ElseValue.ToString(),3);
			KeyArray.SetValue(Properties.Code.ToString(),4);
		}

		public void GetProperty( string Key, ref string Value )
		{		
			if ( Key.Equals( Properties.ColumnName.ToString() ) )
				Value = strColName;
			else if ( Key.Equals( Properties.ReplaceWithList.ToString() ) )
				Value = strReplaceList;
			else if ( Key.Equals( Properties.ElseSetting.ToString() ) )
				Value = Convert.ToInt16(eElseSetting).ToString();
			else if ( Key.Equals( Properties.ElseValue.ToString() ) )
				Value = strElseValue;	
			else if ( Key.Equals( Properties.Code.ToString() ) )
				Value = this.Code;
		}

		public void SetProperty( string Key, string Value )
		{
			if ( Key.Equals( Properties.ColumnName.ToString() ) )
				strColName = Value;
			else if ( Key.Equals( Properties.ReplaceWithList.ToString() ) )			
				strReplaceList = Value;							
			else if ( Key.Equals( Properties.ElseSetting.ToString() ) )
				eElseSetting = (ElseSetting)Convert.ToInt16( Value );
			else if ( Key.Equals( Properties.ElseValue.ToString() ) )
				strElseValue = Value;
			else if ( Key.Equals( Properties.Code.ToString() ) )
				this.Code = Value;
		}

		public void GetSubmitCode( ref string Code )
		{
			Code = this.Code;
		}

		public void GetRunLog( ref string Log )
		{
			Log = "";
		}

		public void GetOptionsFlag( ref int Flags )
		{
			Flags = (int)SASAIOBJLib.APPOPTIONSFLAG.APP_TASK_CATEGORY_DATA;
		}

		public void GetRequiresData( ref int DataRequired )
		{
			DataRequired = (int)SASAIOBJLib.APPDATAOPTIONS.APP_DATAOPTION_REQUIRED;
		}

		public void GetOutputDataCount( ref int count )
		{
			count = 0;
		}

		public void GetOutputDataInfo( int Index, ref string Source, ref string Label, ref int OutputType )
		{
			//does not generate outputdata, so do nothing
		}

		public void GetTaskCategory( ref string Category )
		{			
			//this task is part of the data category,so don't return anything	
		}

		public void GetTaskDescription(ref string Description )
		{
			Description = "Uses Proc SQL to update a Dataset";
		}

		public void GetTaskMajorVersion(ref int MajorVersion)
		{
			MajorVersion = 1;
		
		}

		public void GetTaskMinorVersion(ref int MinorVersion)
		{
			MinorVersion = 0;
		}

		public void GetTaskName(ref string TaskName)
		{
			TaskName = "Update DataSet";
		
		}

		public void GetTaskProcsUsed(ref string Procs)
		{
			Procs = "SQL";
		}

		public void GetTaskType(ref int Type)
		{
			Type = (int) SASAIOBJLib.APPTASKTYPE.APP_TASKTYPE_WIZARD;
		}

		public void GetTaskValidation(ref string Validation)
		{
			Validation = "Not Validated";
		}

	}
}
